@@ -1,5 +1,6 @@
 import time
 import urllib
+from typing import Callable
 from urllib.request import urlretrieve
 from requests.cookies import RequestsCookieJar
 import requests
@@ -156,9 +157,15 @@ def retrieve_file(url: str, path: str, proxies: str = "",
             else:
                 opener.addheaders = [('User-Agent', ua.random)]
             urllib.request.install_opener(opener)
+            bar = None
+            if reporthook:
+                reporthook = reporthook
+            else:
+                bar = process_bar(final_prompt="文件下载完成")
+                reporthook = bar.process
             urlretrieve(url=url, filename=path, reporthook=reporthook, data=data)
-            if need_log:
-                log("文件提取成功")
+            # if need_log:
+            #     log("文件提取成功")
             success = True
             time.sleep(sleep_time)
         except NoProxyException as e:
